/*********************************************************************************
 *      Copyright:  (C) 2024 Avnet. All Rights Reserved.
 *         Author:  Thomas Nizan <tnizan@witekio.com>
 *
 *    Description:  This file is 24AA025 EEPROM driver on AUBoard-15P.
 *
 ********************************************************************************/

#include "eeprom.h"
#include "at24aa025_config.h"

#include "xparameters.h"
#include "xiic.h"
#include "xil_io.h"
#include "xil_printf.h"


#define AT24AA025_IIC_BASE_ADDRESS	XPAR_AXI_IIC_0_BASEADDR
#define AT24AA025_CHIPADDR		0x50


static void print_buf(const char *prompt, uint8_t *buf, int size)
{
    int i;

    if( !buf )
    {
        return ;
    }

    if( prompt )
    {
    	xil_printf("%-32s ", prompt);
    }

    for(i=0; i<size; i++)
    {
    	xil_printf("%02X ", buf[i]);
    }
    xil_printf("\r\n");

    return ;
}

int at24aa025_config_dump(void)
{
	AddressType addr = 0x0;
    u8 buffer[AT24AA025_CONFIG_SIZE];
    int bytes = 0;

    bytes = eeprom_read(AT24AA025_IIC_BASE_ADDRESS, AT24AA025_CHIPADDR, addr, buffer, AT24AA025_CONFIG_SIZE);
    if (bytes != AT24AA025_CONFIG_SIZE)
    {
        xil_printf("error: read operation failed: incorrect number of bytes\r\n");
        return 1;
    }
    print_buf("Current AT24AA025 Config dump:", buffer, AT24AA025_CONFIG_SIZE);
    return 0;
}

static int at24aa025_config_write(u8 *cfg)
{
	int bytes;
	AddressType addr = 0x0;

	bytes = eeprom_write(AT24AA025_IIC_BASE_ADDRESS, AT24AA025_CHIPADDR, addr, cfg, AT24AA025_CONFIG_SIZE);
    if (bytes != AT24AA025_CONFIG_SIZE)
    {
        xil_printf("error: write operation failed: incorrect number of bytes\r\n");
        return 1;
    }

	return 0;
}

static int compare_configs(const u8 *cfg1, const u8 *cfg2, size_t size) {
    for (size_t i = 0; i < size; i++) {
        if (cfg1[i] != cfg2[i]) {
            return 1;
        }
    }

    xil_printf("\r\nVerification done: write operation succeeded\r\n");

    return 0;
}

static void displayWriteConfigurationMessage(void) {
    xil_printf("\r\n");
    xil_printf("*********************************************************\r\n");
    xil_printf("*     You are about to write the following configuration *\r\n");
    xil_printf("*     to the AT24AA025 EEPROM used by the IDT8T49N241    *\r\n");
    xil_printf("*     clock generator.                                  *\r\n");
    xil_printf("*                                                       *\r\n");
    xil_printf("* Please review the configuration below before         *\r\n");
    xil_printf("* proceeding. This will overwrite the existing data     *\r\n");
    xil_printf("* in the EEPROM. Ensure that the new configuration is   *\r\n");
    xil_printf("* correct.                                              *\r\n");
    xil_printf("*********************************************************\r\n");
    xil_printf("\r\n");
}

static int at24aa025_config_verify(u8 *cfg)
{
	int bytes;
    u8 buffer[AT24AA025_CONFIG_SIZE];
	AddressType addr = 0x0;

    bytes = eeprom_read(AT24AA025_IIC_BASE_ADDRESS, AT24AA025_CHIPADDR, addr, buffer, AT24AA025_CONFIG_SIZE);
    if (bytes != AT24AA025_CONFIG_SIZE)
    {
        xil_printf("error: read operation failed: incorrect number of bytes\r\n");
        return 1;
    }
    print_buf("Current AT24AA025 Config dump:", buffer, AT24AA025_CONFIG_SIZE);

    return compare_configs(buffer, cfg, AT24AA025_CONFIG_SIZE);

}

int at24aa025_config_write_operation(void)
{
	int bytes = 0;
    char response;

    u8 *cfg = AT24AA025_IDT8T49N241_CFG;

    displayWriteConfigurationMessage();

    print_buf("AT24AA025 Config to write:", cfg, AT24AA025_CONFIG_SIZE);

    xil_printf("Are you sure you want to proceed? (y/n): ");
    
    response = inbyte();
    outbyte(response);
    xil_printf("\r\n");

    if (response == 'y' || response == 'Y') {
        xil_printf("Proceeding with the write operation...\r\n");
        if (at24aa025_config_write(cfg)) {
            xil_printf("error: write operation failed: write failed\r\n");
            return 1;
        }
        if (at24aa025_config_verify(cfg)) {
            xil_printf("error: write operation failed: verification failed\r\n");
            return 1;
        }
    } else if (response == 'n' || response == 'N') {
        xil_printf("Operation canceled by the user.\r\n");
        return 0;
    } else {
        xil_printf("Invalid input. Please restart the program.\r\n");
        return 0;
    }


    return 0;
}
