/*********************************************************************************
 *      Copyright:  (C) 2024 Avnet. All Rights Reserved.
 *         Author:  Wenxue Guo <wenxue.guo@avnet.com>
 *
 *    Description:  This file is EEPROM driver.
 *
 ********************************************************************************/

#include "eeprom.h"
#include "xparameters.h"
#include "xiic.h"
#include "xil_io.h"
#include "xil_printf.h"

#define EEPROM_PAGESIZE			16
#define EEPROM_SIZE				256

/*+--------------------------------+
 *|     EEPROM read/write API      |
 *+--------------------------------+*/

static int eeprom_pageWrite(UINTPTR BaseAddress, u8 chipAddr, AddressType offset, u8 *data, u32 length)
{
	u8 			WriteBuffer[sizeof(offset) + EEPROM_PAGESIZE];
	unsigned    bytes, SentByteCount;
	u32			CntlReg;

	if( !BaseAddress || !chipAddr || !data || !length || offset>= EEPROM_SIZE)
		return -1;

	if( offset+length > EEPROM_SIZE )
		length = EEPROM_SIZE - offset;

	if( length > EEPROM_PAGESIZE)
		length = EEPROM_PAGESIZE;

    /*
     * A temporary write buffer must be used which contains both the address
     * and the data to be written, put the address in first based upon the
     * size of the address for the EEPROM.
     */
    if (sizeof(AddressType) == 2) {
        WriteBuffer[0] = (u8)(offset >> 8);
        WriteBuffer[1] = (u8)(offset);
    } else if (sizeof(AddressType) == 1) {
        WriteBuffer[0] = (u8)(offset);
    }

    /* Put the data in the write buffer following the address */
    memcpy(&WriteBuffer[sizeof(AddressType)], data, length);

    /*
     * Set the address register to the specified address by writing
     * the address to the device, this must be tried until it succeeds
     * because a previous write to the device could be pending and it
     * will not ack until that write is complete.
     */
    do {
    	bytes = XIic_Send(BaseAddress, chipAddr, (u8 *)&offset, sizeof(offset), XIIC_STOP);
        if (bytes != sizeof(offset)) {
            /* Send is aborted so reset Tx FIFO */
            CntlReg = XIic_ReadReg(BaseAddress, XIIC_CR_REG_OFFSET);
            XIic_WriteReg(BaseAddress, XIIC_CR_REG_OFFSET, CntlReg | XIIC_CR_TX_FIFO_RESET_MASK);
            XIic_WriteReg(BaseAddress, XIIC_CR_REG_OFFSET, XIIC_CR_ENABLE_DEVICE_MASK);
        }
    } while (bytes != sizeof(offset));


    /* Write a page of data at the specified address to the EEPROM. */
    SentByteCount = XIic_Send(BaseAddress, chipAddr, WriteBuffer, sizeof(offset)+length, XIIC_STOP);

    /*
     * Wait for the write to be complete by trying to do a write and
     * the device will not ack if the write is still active.
     */
    do {
    	bytes = XIic_Send(BaseAddress, chipAddr, (u8 *)&offset, sizeof(offset), XIIC_STOP);
        if (bytes != sizeof(offset)) {
            /* Send is aborted so reset Tx FIFO */
            CntlReg = XIic_ReadReg(BaseAddress, XIIC_CR_REG_OFFSET);
            XIic_WriteReg(BaseAddress, XIIC_CR_REG_OFFSET, CntlReg | XIIC_CR_TX_FIFO_RESET_MASK);
            XIic_WriteReg(BaseAddress, XIIC_CR_REG_OFFSET, XIIC_CR_ENABLE_DEVICE_MASK);
        }
    } while (bytes != sizeof(offset));

    /* Return the number of bytes written to the EEPROM */
    return SentByteCount - sizeof(offset);
}

int eeprom_write(UINTPTR BaseAddress, u8 chipAddr, AddressType offset, u8 *data, u32 length)
{
	unsigned    bytes;
	unsigned    left;

	if( !BaseAddress || !chipAddr || !data || !length || offset>= EEPROM_SIZE)
		return -1;

	if( offset+length > EEPROM_SIZE )
		length = EEPROM_SIZE - offset;

	/* Write max one page at a time */
	left = length;
	while(left > 0)
	{
		bytes = left>EEPROM_PAGESIZE ? EEPROM_PAGESIZE : left;
		if( bytes != eeprom_pageWrite(BaseAddress, chipAddr, offset, data, bytes) )
		{
			return -2;
		}

		left   -= bytes;
		data   += bytes;
		offset += bytes;
	}

	return length;
}

int eeprom_read(UINTPTR BaseAddress, u8 chipAddr, AddressType offset, u8 *buf, u32 size)
{
	unsigned    bytes;
	u16			StatusReg;
	u32			CntlReg;

	if( !BaseAddress || !chipAddr || !buf || !size )
		return -1;

	memset(buf, 0, size);

    /*
     * Set the address register to the specified address by writing
     * the address to the device, this must be tried until it succeeds
     * because a previous write to the device could be pending and it
     * will not ack until that write is complete.
     */
    do {
        StatusReg = XIic_ReadReg(BaseAddress, XIIC_SR_REG_OFFSET);
        if (!(StatusReg & XIIC_SR_BUS_BUSY_MASK))
        {
        	bytes = XIic_Send(BaseAddress, chipAddr, &offset, sizeof(AddressType), XIIC_STOP);
            if (bytes != sizeof(AddressType))
            {
                /* Send is aborted so reset Tx FIFO */
                CntlReg = XIic_ReadReg(BaseAddress, XIIC_CR_REG_OFFSET);
                XIic_WriteReg(BaseAddress, XIIC_CR_REG_OFFSET, CntlReg | XIIC_CR_TX_FIFO_RESET_MASK);
                XIic_WriteReg(BaseAddress, XIIC_CR_REG_OFFSET, XIIC_CR_ENABLE_DEVICE_MASK);
            }
        }
    } while (bytes != sizeof(AddressType));

    /* Read the number of bytes at the specified address from the EEPROM */
    bytes = XIic_Recv(BaseAddress, chipAddr, buf, size, XIIC_STOP);

    /* Return the number of bytes read from the EEPROM */
    return bytes;
}
